#ifdef CONFIG_LIDS
#ifndef LIDS_H
#define LIDS_H

#include <linux/kernel.h>
#include <linux/sysctl.h>
#include <linux/malloc.h>
#include <linux/lidsext.h>
#include <linux/tty.h>

#define LIDS_VERSION	"0.9.15 for 2.2.19"

#define LIDS_FLAGS_LIDS_ON            1
#define LIDS_FLAGS_RELOAD_CONF        4      
#define LIDS_FLAGS_LIDS_LOCAL_ON      8

#define LIDS_MAX_DOMAIN		64	/* max exec domain for a process */
/*
 *	ACL target.
 */ 

#define LIDS_DENY	0	/* DENY ACCESS*/
#define LIDS_READONLY 	1	/* Read Only File */
#define LIDS_APPEND	2	/* APPEND ONLY FILE */
#define LIDS_WRITE	4	/* Protect MBR Writing to device */
#define LIDS_IGNORE	8	/* Ignore the protection */
#define LIDS_CAP	16	/* object is inheritable */


/* 
 * Me ? Paranoiac !?
 *
 * The magic numbers are all around the encrypted password.
 * They have a null byte to bother ASCIIZ functions.
 */

#define LIDS_MAGIC_1 0x004e6748
#define LIDS_MAGIC_2 0x68002d68
#define LIDS_MAGIC_3 0xe68400c6
#define LIDS_MAGIC_4 0xd94aac00

#define LIDS_FISSET(fs,f) ((fs & f) != 0)
#define LIDS_FISCLR(fs,f) (!(fs & f))
#define LIDS_FSET(fs,f) (fs |= f)
#define LIDS_FCLR(fs,f) (fs &= ~f)

typedef char passwd_t[64];

typedef struct lids_locks_s {
	int magic1;
	kernel_cap_t cap_bset;
	int magic2;
	lids_flags_t flags;
	int magic3;
	passwd_t passwd;
	int magic4;
} lids_locks_t;

struct secure_ino {
	unsigned long int ino;		/* the inode number */
	kdev_t 	dev;			/* the dev number */
	int 	type;			/* the file type */
};

struct allowed_ino {
	unsigned long int ino;		/* the inode number */
	kdev_t 	dev;			/* the dev number */
};
/* use in task struct to represent the acl */
struct lids_acl {
	struct lids_acl *next;
	unsigned long int ino;
	kdev_t	dev;
	int	type; /* R W A Deny*/
	int	inherit; /* the inherit level (TTL) */
};
/* lids_domain define the process's exec domain*/
struct lids_domain {
	int 	counter;
	struct dentry dentry[LIDS_MAX_DOMAIN];
};
/* save all the system defined acl here */
struct lids_sys_acl {
	unsigned long int ino;	/* the subject node number */
	unsigned long flags;	/* capability flags */
	int cap_inherit[32];	/* inherit array */
	int forked;		/* forked? */
	struct lids_acl *lids_acl;/* object acl */
	struct lids_acl *lids_domain; /* restricted exec domain */
	kdev_t 	dev;		/* the subject dev number */
};

/* FIXME: some more externals in kernel/signal.c and kernel/sysctl.c */

extern int lids_load;		/* 1 = load ids protection , 0 = don't load */
extern lids_flags_t lids_flags;	/* 1 = load ids protection , 0 = don't load */
extern int lids_local_on;
extern unsigned long lids_current;

extern int lids_proc_locks_sysctl(ctl_table *table, int write, struct file *filp, void *buffer, size_t *lenp, int conv, int op);
extern int lids_init(void);
extern struct lids_sys_acl * lids_search_acl(unsigned long int ino,kdev_t dev,unsigned long lids_curr);
extern int lids_check_base(struct dentry *base, int flag);
extern int lids_set_acls(struct lids_sys_acl *);
extern int lids_check_hidden_inode(unsigned long int ino,kdev_t dev);
extern int lids_local_off(void);
#ifdef CONFIG_LIDS_SA_THROUGH_NET
extern int lids_klids_init(void);
#endif
#ifdef CONFIG_LIDS_PORT_SCAN_DETECTOR
extern void lids_port_scanner_detector_init(void);
#endif

#endif /* LIDS_H */
#endif /* CONFIG_LIDS */
