#ifdef CONFIG_LIDS
#ifndef LIDSEXT_H
#define LIDSEXT_H

#ifdef CONFIG_LIDS_DEBUG
#define LIDS_DEBUG
#endif

#ifdef LIDS_DEBUG
#define LIDS_DBG(msg...)  printk(KERN_DEBUG "LIDS." __FUNCTION__ ": " ##msg)
#else
#define LIDS_DBG(msg...)
#endif
extern void lids_cap_log(int );

#define lids_get_current_dentry(dentry)                          \
        {                                                        \
	    struct vm_area_struct * vma;                         \
	    dentry = NULL;                                       \
	    if (current->mm) {                                   \
		vma = current->mm->mmap;                         \
		while (vma) {                                    \
		    if ((vma->vm_flags & VM_EXECUTABLE) &&       \
			vma->vm_file) {                          \
			dentry=vma->vm_file->f_dentry;           \
			break;                                   \
		    }                                            \
		    vma = vma->vm_next;                          \
		}                                                \
	    }                                                    \
	}
            

#define lids_info_params1 f_dentry->d_iname,                     \
                     MAJOR(f_dentry->d_inode->i_dev),            \
                     MINOR(f_dentry->d_inode->i_dev),            \
                     f_dentry->d_inode->i_ino,                   \
                     current->pid,                               \
                     current->uid,                               \
                     current->gid

#define lids_info_params2 current->pid,                          \
                     current->uid,                               \
                     current->gid

#ifdef CONFIG_LIDS_HANGUP
#define lids_hangup_console()    \
do{	\
     if(current->tty != NULL && !lids_first_time ) {      \
		if (!test_bit(TTY_LIDS_NO_HANGUP,&current->tty->flags) ) \
                	tty_vhangup(current->tty);      \
        }	\
}while(0)
#else
#define lids_hangup_console() do {} while (0)
#endif


#ifdef CONFIG_LIDS_SA_THROUGH_NET
#define lids_print(message, args...) sprintf(p, message , ## args)
#else
#define lids_print(message, args...) printk(KERN_ALERT message , ## args)
#endif


#define lids_make_alert(message, args...)                        \
     if (f_dentry && f_dentry->d_inode) {                        \
        if (current->tty) {                                      \
             lids_print( "LIDS: %s (%d %d inode %ld) pid %d user (%d/%d) on %s%d: " message "\n", \
                        lids_info_params1,                       \
                        current->tty->driver.name,               \
		        ( MINOR((current->tty)->device)-         \
		        current->tty->driver.minor_start+        \
		        current->tty->driver.name_base ) ,       \
		        ## args);                                \
        }                                                        \
        else {                                                   \
             lids_print( "LIDS: %s (%d %d inode %ld) pid %d user (%d/%d) on NULL tty: " message "\n", \
             lids_info_params1 ,                                 \
             ## args);                                           \
        }                                                        \
     }                                                           \
     else {                                                      \
        if (current->tty) {                                      \
             lids_print( "LIDS: (undetermined program) pid %d user (%d/%d) on %s%d: " message "\n",  \
                        lids_info_params2,                       \
                        current->tty->driver.name,               \
	    	        ( MINOR((current->tty)->device)-         \
		        current->tty->driver.minor_start+        \
		        current->tty->driver.name_base ) ,       \
		        ## args);                                \
        }                                                        \
        else {                                                   \
             lids_print( "LIDS: (undetermined program) pid %d user (%d/%d) on NULL tty: " message "\n", \
                        lids_info_params2 ,                      \
             ## args);                                           \
        }                                                        \
     }

extern void lids_send_message(char *msg,int len);


#ifdef CONFIG_LIDS_SA_THROUGH_NET
#define lids_log(message, args...) {                               \
                             char *p;                              \
                             p=(char *)kmalloc(2048,GFP_KERNEL);   \
                             lids_make_alert(message , ## args);   \
                             printk(KERN_ALERT "%s",p);            \
                             lids_send_message(p,strlen(p));       \
                      }
#else
#define lids_log(message, args...) lids_make_alert(message , ## args)
#endif


#ifdef CONFIG_LIDS_NO_FLOOD_LOG
#define lids_security_alert(message, args...)                                                 \
({                                                                                            \
       static unsigned long warning_time = 0, no_flood_yet = 0;                               \
       static spinlock_t lids_security_alert_lock = SPIN_LOCK_UNLOCKED;                       \
       struct dentry *f_dentry;                                                               \
                                                                                              \
       lids_get_current_dentry(f_dentry);                                                     \
       spin_lock(&lids_security_alert_lock);                                                  \
                                                                                              \
/* Make sure at least CONFIG_LIDS_TIMEOUT_AFTER_FLOOD passed since the last warning logged */ \
       if (!warning_time || jiffies - warning_time > CONFIG_LIDS_TIMEOUT_AFTER_FLOOD * HZ) {  \
               warning_time = jiffies; no_flood_yet = 1;                                      \
               lids_log(message , ## args);                                                   \
       } else if (no_flood_yet) {                                                             \
               warning_time = jiffies; no_flood_yet = 0;                                      \
               lids_log("more " message                                                 \
                       ",logging disabled for %i seconds" ,                                 \
                       ## args ,                                                              \
                       CONFIG_LIDS_TIMEOUT_AFTER_FLOOD);                                      \
       }                                                                                      \
                                                                                              \
       spin_unlock(&lids_security_alert_lock);                                                \
	lids_hangup_console();    \
})
#else /* CONFIG_LIDS_NO_FLOOD_LOG */
#define lids_security_alert(message, args...)                                                 \
({                                                                                            \
       struct dentry *f_dentry;                                                               \
       static spinlock_t lids_security_alert_lock = SPIN_LOCK_UNLOCKED;                       \
                                                                                              \
       spin_lock(&lids_security_alert_lock);                                                  \
       lids_get_current_dentry(f_dentry);                                                     \
       lids_log(message , ## args);                                                           \
       spin_unlock(&lids_security_alert_lock);                                                \
	lids_hangup_console();    \
})
#endif /* CONFIG_LIDS_NO_FLOOD_LOG */

#ifdef CONFIG_LIDS_ALLOW_SWITCH
#define lids_local_load ( lids_local_on || (!lids_local_off()) )
#else
#define lids_local_load 1
#endif /* CONFIG_LIDS_ALLOW_SWITCH */

typedef int lids_flags_t;


extern int lids_local_off(void);
extern int lids_reload_conf;
extern int lids_load;
extern int lids_local_on;
extern int lids_local_pid;
extern int lids_first_time;
extern lids_flags_t lids_flags;



#endif /* LIDSEXT_H */
#endif /* CONFIG_LIDS */
